@@ -3,7 +3,7 @@
 # Simple Backup suit
 #
 # Running this command will upgrade a backup directory to latest format.
-# This is also a backend for simple-restore-gnome GUI.
+# This is also a backend for simple-restore-gnome GUI and sbackupd.
 #
 # Author: Aigars Mahinovs <aigarius@debian.org>
 #
@@ -24,6 +24,7 @@
 import tarfile, sys, re, os, os.path, shutil, datetime, filecmp, gzip, tempfile, zlib
 import cPickle as pickle
 import gettext
+import traceback
 from gettext import gettext as _
 try:
     import gnomevfs
@@ -33,9 +34,6 @@ except ImportError:
 
 class SBUpgrade:
 	def upgrade_target( self, target, purge=0 ):
-		if self.permissions( target )%2**9 != 0700:
-			print _("I: Securing target directory at: %s") % target
-			self.chmod( target, 0700 )
 
 		r = re.compile(r"^(\d{4})-(\d{2})-(\d{2})_(\d{2})[\:\.](\d{2})[\:\.](\d{2})\.\d+\..*?\.(.+)$")
 
@@ -53,6 +51,8 @@ class SBUpgrade:
 		listing.reverse()
 
 		for adir in listing:
+			if not self.isdir( target+"/"+adir ):
+				continue
 			if ":" in adir:
 				print "I: Renaming directory: '"+adir+"' to '"+adir.replace( ":", "." )+"'"
 				self.rename( target+"/"+adir, adir.replace( ":", "." ) )
@@ -65,28 +65,42 @@ class SBUpgrade:
 	def upgrade_tdir( self, tdir ):
 		good = True
 
-		if self.permissions( tdir )%2**9 != 0700 or self.permissions( tdir+"/files.tgz" )%2**9 != 0600:
-			print _("I: Securing permissions at: %s") % tdir
-			self.perm_secure( tdir )
-			good = False
-			
 		if self.exists( tdir+"/tree") and not self.exists( tdir+"/flist" ):
 			print _("I: Upgrading from v1.0 to v1.2: %s") % tdir
 			self.upgrade_v1( tdir )
 			good = False
 
-		v = self.openfile( tdir + "/ver" )
-		if not v.read(5) == "1.3\n":
-			print _("I: Upgradeing to v1.3: %s") % tdir
-			self.upgrade_v13( tdir )
-			good = False
-		v.close()
-
-		if self.exists( tdir+"/flist" ) and self.exists( tdir+"/files.tgz" ):
+		if self.exists(tdir + "/ver") :
+			ver = self.readfile( tdir + "/ver" )
+		else : 
+			print ("W: Error reading %s/ver ! Ignoring incomplete or non-backup directory." % tdir)  
 			return
 		
+		try : 
+			major = int(ver[0])
+			minor = int(ver[2])
+		except Exception, e:
+			print ("W: %s/ver doesn't contain valid value ! Ignoring incomplete or non-backup directory." % tdir)
+			print ("Error info : ")
+			traceback.print_exc()
+			self.delete(tdir+"/ver")
+			return 
+				
+		if major < 2 and minor < 3:
+			print _("I: Upgrading to v1.3: %s") % tdir
+			self.upgrade_v13( tdir )
+			good = False
+			ver = "1.3\n"
 
-		print "W: Unknown or incomplete directory format at: ", tdir
+
+		if int(ver[0]) < 2 and int(ver[2:]) < 4:
+			print _("I: Upgrading to v1.4: %s") % tdir
+			self.upgrade_v14( tdir )
+			good = False
+
+		if self.exists( tdir+"/flist" ) and self.exists( tdir+"/fprops" ) and self.exists( tdir+"/files.tgz" ) and self.exists( tdir+"/ver" ):
+			return
+		
 
 	def upgrade_v1( self, tdir ):
 		i = self.openfile(tdir+"/tree")
@@ -100,13 +114,14 @@ class SBUpgrade:
 		n.close()
 		i.close()
 		v = self.openfile( tdir+"/ver", True )
-		v.write("1.3\n")
+		v.write("1.4\n")
 		v.close()
 		
 	def upgrade_v13( self, tdir ):
-		flist = gnomevfs.read_entire_file( tdir+"/flist" ).split( "\n" )
-		fprops = gnomevfs.read_entire_file( tdir+"/fprops" ).split( "\n" )
-		if len(flist)==len(fprops) and len(flist) > 1:
+		try:
+		    flist = gnomevfs.read_entire_file( tdir+"/flist" ).split( "\n" )
+		    fprops = gnomevfs.read_entire_file( tdir+"/fprops" ).split( "\n" )
+		    if len(flist)==len(fprops) and len(flist) > 1:
 			l = self.openfile(tdir+"/flist.v13", True)
 			p = self.openfile(tdir+"/fprops.v13", True)
 			for a,b in zip(flist,fprops):
@@ -114,37 +129,51 @@ class SBUpgrade:
 				p.write( b+"\000" )
 			l.close()
 			p.close()
-		else:
-			print _("W: Damaged backup metainfo - file with newline detected: %s") % tdir
-			print _("I: Recovering file info ... this can take some time.")
-			t = self.openfile(tdir+"/files.tgz")
-			a = tarfile.open( "dummy", "r:gz", t )
-			files = a.getmembers()
-			l = self.openfile( tdir+"/flist.v13", True )
-			p = self.openfile( tdir+"/fprops.v13", True )
-			for file in files:
-			    l.write( file.name + "\000" )
-			    p.write( str(file.mode)+str(file.uid)+str(file.gid)+str(file.size)+str(file.mtime)+"\000")
-			print _("I: Recovery complete.")
-		self.rename(tdir+"/flist", "flist.old")
-		self.rename(tdir+"/flist.v13", "flist")
-		self.rename(tdir+"/fprops", "fprops.old")
-		self.rename(tdir+"/fprops.v13", "fprops")
-		v = self.openfile( tdir+"/ver", True )
-		v.write("1.3\n")
-		v.close()
+		    else:
+			print _("W: Damaged backup metainfo - disabling %s") % tdir
+			if self.exists(tdir+"/ver") : self.delete(tdir+"/ver")
+			
+		    self.rename(tdir+"/flist", "flist.old")
+		    self.rename(tdir+"/flist.v13", "flist")
+		    self.rename(tdir+"/fprops", "fprops.old")
+		    self.rename(tdir+"/fprops.v13", "fprops")
+		    v = self.openfile( tdir+"/ver", True )
+		    v.write("1.3\n")
+		    v.close()
+		except:
+		    print _("W: Damaged backup metainfo - disabling %s") % tdir
+		    if self.exists(tdir+"/ver") : self.delete(tdir+"/ver")
+		    
 		
 
+	def upgrade_v14( self, tdir ):
+		self.delete( tdir + "/ver" )
+		
+		if not self.exists( tdir+"/flist" ):
+			return False
+		if not self.exists( tdir+"/fprops" ):
+			return False
+		if not self.exists( tdir+"/files.tgz" ):
+			return False
+		if not self.exists( tdir+"/excludes" ):
+			return False
+		
+		
+		v = self.openfile( tdir+"/ver", True )
+		v.write("1.4\n")
+		v.close()
+
+
 	def perm_secure( self, tdir ):
-		self.chmod( tdir, 0700 )
-		self.chmod( tdir+"/ver", 0600 )
-		self.chmod( tdir+"/tree", 0600 )
-		self.chmod( tdir+"/flist", 0600 )
-		self.chmod( tdir+"/fprops", 0600 )
-		self.chmod( tdir+"/files.tgz", 0600 )
-		self.chmod( tdir+"/packages", 0600 )
-		self.chmod( tdir+"/excludes", 0600 )
-		self.chmod( tdir+"/base", 0600 )
+		self.chmod( tdir, 0750 )
+		self.chmod( tdir+"/ver", 0640 )
+		self.chmod( tdir+"/tree", 0640 )
+		self.chmod( tdir+"/flist", 0640 )
+		self.chmod( tdir+"/fprops", 0640 )
+		self.chmod( tdir+"/files.tgz", 0640 )
+		self.chmod( tdir+"/packages", 0640 )
+		self.chmod( tdir+"/excludes", 0640 )
+		self.chmod( tdir+"/base", 0640 )
 
 	def purge( self, target, listing, purge ):
 		r = re.compile(r"^(\d{4})-(\d{2})-(\d{2})_(\d{2})[\:\.](\d{2})[\:\.](\d{2})\.\d+\..*?\.(.+)$")
@@ -229,52 +258,47 @@ class SBUpgrade:
 
 	def delete( self, uri ):
 		if self.islocal( uri ):
-		    shutil.rmtree( uri, False )
-		    return True
+			if self.isdir( uri ):
+				shutil.rmtree( uri, False )
+				return True
+			else : 
+				os.unlink(uri)
+				return True
 		else:
-		    try:
 			if not self.isdir( uri ):
 			    gnomevfs.unlink( uri )
 			else:
-			    d = gnomevfs.open_directory( uri )
-			    for f in d:
-				if f.name=="." or f.name=="..":
-				    continue
-				if f.type==2:
-				    self.delete( uri+"/"+f.name )
-				else:
-				    gnomevfs.unlink( uri+"/"+f.name )
-			    d.close()
-			    gnomevfs.remove_directory( uri )
-		    except: return False
+				d = gnomevfs.open_directory( uri )
+				for f in d:
+					if f.name=="." or f.name=="..":
+						continue
+					if f.type==2:
+						self.delete( uri+"/"+f.name )
+					else:
+						gnomevfs.unlink( uri+"/"+f.name )
+				gnomevfs.remove_directory( uri )
 
 	def isdir( self, uri ):
-		try:	return ( gnomevfs.get_file_info( uri ).type == 2 )
-		except:	return False
+		try :
+			return ( gnomevfs.get_file_info( uri ).type == 2 )
+		except Exception, e : 
+			print "W: " +str(e)
+			return False
 
 	def rename( self, uri, name ):
-		try:
-			p = gnomevfs.get_file_info( uri )
-			p.name = name
-			gnomevfs.set_file_info( uri, p, 1 )
-		except:
-			return False
+		p = gnomevfs.get_file_info( uri )
+		p.name = name
+		gnomevfs.set_file_info( uri, p, 1 )
+
 				
 
 	def chmod( self, uri, mode ):
-		try:
-			p = gnomevfs.get_file_info( uri )
-			p.permissions = mode
-			gnomevfs.set_file_info( uri, p, 2 )
-		except:
-			return False
+		p = gnomevfs.get_file_info( uri )
+		p.permissions = mode
+		gnomevfs.set_file_info( uri, p, 2 )
 
 	def permissions( self, uri ):
-		try:
-			p = gnomevfs.get_file_info( uri ).permissions
-		except:
-			p = False
-		return p
+		return gnomevfs.get_file_info( uri ).permissions
 
 	def exists( self, uri ):
 		if self.islocal(uri):
@@ -283,13 +307,7 @@ class SBUpgrade:
 			return gnomevfs.exists( uri )
 
 	def islocal( self, uri ):
-                local = True
-                try:
-                    if not gnomevfs.URI( uri ).is_local:
-                        local = False
-                except:
-                    pass
-		return local
+		return gnomevfs.URI( uri ).is_local
 
 	def openfile( self, uri, write=False ):
 		if self.islocal( uri ):
@@ -305,6 +323,17 @@ class SBUpgrade:
 					return gnomevfs.create( uri, 2 )
 			else:
 				return gnomevfs.open( uri, 1 )
+				
+	def readfile(self, uri) :
+		" Read a file from a given URI and return a string with the read content "
+		if self.islocal( uri ) :
+			f = open( uri, "r" )
+			value = f.read()
+			f.close()
+			return str( value )
+		else :
+			return str( gnomevfs.read_entire_file( uri ) )
+		
 
 
 if __name__ == "__main__":
